@@ -13,13 +13,22 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-from functools import wraps
 
 from itertools import tee
-from functools import total_ordering
+from functools import total_ordering, wraps
+import warnings
 from contextlib import contextmanager
 import re
 import sys
+from types import StringTypes
+import os
+import fcntl
+from os import path
+import time
+from abc import ABCMeta, abstractmethod
+from types import GeneratorType
+from math import sqrt
+
 
 try:
     import json as json_module
@@ -94,6 +103,7 @@ class Sizes(object):
     Gb = gigabyte = Mb * 1000
     T = TiB = tibibyte = G * 1024
     Tb = terabyte = Gb * 1000
+    Sector = 512
 
     def __getattr__(self, name):
         if name.startswith('from_'):
@@ -103,6 +113,11 @@ class Sizes(object):
         else:
             raise AttributeError
 
+    def _from(self, name, count):
+        return count * getattr(Sizes, name)
+
+    def _to(self, name, count):
+        return count / getattr(Sizes, name)
 
 def imap_regexp(re_compiled, l, whole=False):
     """
@@ -142,6 +157,9 @@ def cached(each_instance=False):
 
 
 class Singleton(type):
+    """
+    Существует только один экземпляр данного класса
+    """
     _instances = {}
 
     def __call__(cls, *args, **kw):
@@ -152,6 +170,10 @@ class Singleton(type):
 
 
 class SingletonParam(type):
+    """
+    Параметризированный Singleton: может быть создано множество объектов одного
+    и того же класса, но только если у них разные параметры инициализации
+    """
     def __init__(cls, name, bases, dicts):
         super(SingletonParam, cls).__init__(name, bases, dicts)
         cls.instance = {}
@@ -197,6 +219,11 @@ def classificate(iterator):
 
 
 def debug(prefix="DEBUG {name}({args},{kw})={ret}"):
+    """
+    Декоратор для вывода отладочных сообщений при вызове функции
+    :param prefix:
+    :return:
+    """
     def debug_decor(f):
         @wraps(f)
         def wrapper(*args, **kw):
@@ -233,18 +260,17 @@ class ClassPropertyDescriptor(object):
 
 
 def classproperty(func):
+    """
+    Классовое свойство
+    :param func:
+    :return:
+    """
     if not isinstance(func, (classmethod, staticmethod)):
         func = classmethod(func)
 
     return ClassPropertyDescriptor(func)
 
 
-import os
-import fcntl
-from os import path
-import time
-
-
 class LockError(Exception):
     pass
 
@@ -358,7 +384,7 @@ def max_default(iterable, key=lambda x: x, default=None):
         return default
 
 
-def traverse(o, tree_types=(list, tuple)):
+def traverse(o, tree_types=(list, tuple, GeneratorType)):
     """
     Вернуть последовательно все элемены списка, "распаковов" встроенные
 
@@ -405,6 +431,13 @@ def unique(iterable):
         seen[x] = 1
         yield x
 
+def sorteduniqresult(f):
+    @wraps(f)
+    def wrapper(*args, **kw):
+        for x in sorted(set(f(*args, **kw))):
+            yield x
+    return wrapper
+
 def repeater(*args):
     """
     Циклы для повторяющихся действий с ожидаением
@@ -419,8 +452,8 @@ def repeater(*args):
 
 def iterate_list(l):
     """
-    Функция позволяет обрабатывать поэлементно списки, кортежи, исключая
-    последовательности, которые ими не являются
+    Перебрать последовательность (исключая строки) поэлемено,
+    или же возвращает последовательность из одно элемента
 
     l1 = 1
     l2 = [1,2]
@@ -430,7 +463,7 @@ def iterate_list(l):
     :param l: Список, кортеж, или прочее
     :return:
     """
-    if isinstance(l, (list, tuple)):
+    if not isinstance(l, StringTypes):
         for x in l:
             yield x
     else:
@@ -445,6 +478,19 @@ def has_any(a, b):
     """
     return any(x in a for x in b)
 
+def deprecated(func):
+    """This is a decorator which can be used to mark functions
+    as deprecated. It will result in a warning being emmitted
+    when the function is used."""
+
+    @wraps(func)
+    def new_func(*args, **kwargs):
+        warnings.simplefilter('always', DeprecationWarning) #turn off filter 
+        warnings.warn("Call to deprecated function {}.".format(func.__name__), category=DeprecationWarning, stacklevel=2)
+        warnings.simplefilter('default', DeprecationWarning) #reset filter
+        return func(*args, **kwargs)
+
+    return new_func
 
 def unpack_single_opts(args):
     """
@@ -462,3 +508,115 @@ def unpack_single_opts(args):
             yield arg
 
 drop_slash = lambda x: x[1:] if x.startswith('/') else x
+
+class Cachable(object):
+    """
+    Объект содержащий кэширующие методы
+    """
+    def __init__(self):
+        self.clear_method_cache()
+
+    def clear_method_cache(self):
+        self._method_cache = {}
+
+    @staticmethod
+    def methodcached(key=lambda *args, **kw: hash(args)):
+        """
+        Кэширование результата по параметрам метода, по ключевой функции
+        можно выбрать какие параметры играют роль
+        """
+        def decorator(f):
+            funcname = f.func_name
+
+            @wraps(f)
+            def wrapper(self, *args, **kw):
+                keyval = key(*args, **kw)
+                assert isinstance(self, Cachable)
+                if funcname not in self._method_cache:
+                    self._method_cache[funcname] = {}
+                cache = self._method_cache[funcname]
+                if keyval not in cache:
+                    cache[keyval] = f(self, *args, **kw)
+                return cache[keyval]
+
+            @wraps(f)
+            def null_wrapper(self):
+                assert isinstance(self, Cachable)
+                if funcname not in self._method_cache:
+                    self._method_cache[funcname] = f(self)
+                return self._method_cache[funcname]
+            if len(f.func_code.co_varnames) > 1:
+                return wrapper
+            else:
+                return null_wrapper
+
+        return decorator
+
+def firstelement(i, fallback=''):
+    for x in i:
+        return x
+    return fallback
+
+class GenericFs(object):
+    """
+    Объект обобщения файловых систем
+    """
+    __metaclass__ = ABCMeta
+
+    @abstractmethod
+    def exists(self, fn):
+        pass
+
+    @abstractmethod
+    def read(self, fn):
+        pass
+
+    @abstractmethod
+    def glob(self, fn):
+        pass
+
+    @abstractmethod
+    def realpath(self, fn):
+        pass
+
+    @abstractmethod
+    def write(self, fn, data):
+        pass
+
+    @abstractmethod
+    def listdir(self, dn, fullpath=False):
+        pass
+
+class Brokenable(object):
+    @property
+    def broken(self):
+        return False
+
+    @staticmethod
+    def broken_result(res=None):
+        def decor(f):
+            @wraps(f)
+            def wrapper(self, *args, **kw):
+                if self.broken:
+                    return res
+                return f(self, *args, **kw)
+            return wrapper
+        return decor
+
+
+def get_best_nearest_resolution(preferred_res, support_res, aspect_only=False):
+    """
+    Получить наилучшее ближайшее к preferred_res разрешенение из support_res.
+    """
+    width, height = map(int, preferred_res.split('x'))
+    gep = sqrt(height ** 2 + width ** 2)
+    k = float(width) / float(height)
+    support_res_int = (tuple(map(int, x.split("x"))) for x in support_res)
+    if not aspect_only:
+        support_res_int = [(x,y) for x,y in support_res_int if x <= width and y <= height]
+        if not support_res_int:
+            return None
+    bestRes = min(support_res_int,
+                  key=lambda x: (abs(x[0] / float(x[1]) - k),
+                                 abs(gep - sqrt(x[0] ** 2 + x[1] ** 2))))
+    return "%sx%s" % bestRes
